/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "handlebox.h"

#include <papyrus/circle.h>

namespace Papyrus
{

  Handlebox::Handlebox(Handles handles) : Boxed()
  {
    Rectangle::pointer rectangle = Rectangle::create( 7, 7 );
    rectangle->set_fill( Cairo::SolidPattern::create_rgba( 0.0, 1.0, 0.0, 1.0 ) );
    rectangle->set_outline( Cairo::SolidPattern::create_rgba( 0.0, 0.0, 0.0, 1.0 ) );
    switch ( handles ) {
      case NO_HANDLES:
        break;
      case CORNER_HANDLES:
        this->set_corner_handle_shape( rectangle );
        break;
      case ALL_HANDLES:
        this->set_handle_shape( rectangle );
        break;
    }
  }

  Handlebox::~Handlebox()
  {}

  Handlebox::pointer Handlebox::create(Handles handles)
  {
    return Handlebox::pointer( new Handlebox( handles ) );
  }

  void Handlebox::update_extents()
  {
    double x, y, w, h, offset = 0.0, offset_2, offset_4;
    this->rebuild_extents();
    this->extents().xywh( x, y, w, h );
    if ( m_box->use_outline() )
      offset = m_box->outline().width();
    offset_2 = 2.0 * offset;
    offset_4 = 4.0 * offset;
    m_box->set_xywh( x - offset, y - offset, w + offset_2, h + offset_2 );

    double left = x - offset,
           right = x + w + offset,
           hcenter = x + w / 2.0,
           top = y - offset,
           bottom = y + h + offset,
           vcenter = y + h / 2.0;

    this->position_handle( m_handle_shape[ 0 ], hcenter, top );
    this->position_handle( m_handle_shape[ 1 ], right, top );
    this->position_handle( m_handle_shape[ 2 ], right, vcenter );
    this->position_handle( m_handle_shape[ 3 ], right, bottom );
    this->position_handle( m_handle_shape[ 4 ], hcenter, bottom );
    this->position_handle( m_handle_shape[ 5 ], left, bottom );
    this->position_handle( m_handle_shape[ 6 ], left, vcenter );
    this->position_handle( m_handle_shape[ 7 ], left, top );

    Region region( x - offset_2, y - offset_2, w + offset_4, h + offset_4 );
    for ( int i = 0; i < 8; i++ )
      if ( m_handle_shape[ i ] )
        region.include( m_handle_shape[ i ] ->global_extents() );
    this->set_extents( region );
    this->need_redraw();
  }

  void Handlebox::draw( Cairo::RefPtr< Cairo :: Context > cairo )
  {
    if ( m_children.size() == 0 )
      return;
    
    Boxed::draw( cairo );
    for ( int i = 0; i < 8; i++ )
      if ( m_handle_shape[ i ] )
        m_handle_shape[ i ] ->render( cairo );
  }

  void Handlebox::set_handle_shape( Drawable::pointer shape )
  {
    Group::pointer group;
    for ( int i = 0; i < 8; i++ ) {
//       group = Group::create();
      m_handle_shape[ i ] = /*group;
      group->add(*/ shape->clone() /*)*/;
    }
  }

  void Handlebox::set_handle_shape( unsigned short int which, Drawable::pointer shape )
  {
    if ( which > 7 )
      return ;
    m_handle_shape[ which ] = shape;
  }

  void Handlebox::set_corner_handle_shape( Drawable::pointer shape )
  {
    Group::pointer group;
    for ( int i = 1; i < 8; i += 2 ) {
//       group = Group::create();
      m_handle_shape[ i ] = /*group;
      group->add(*/ shape->clone() /*)*/;
    }
  }

  Drawable::pointer Handlebox::get_handle_shape( unsigned short int which )
  {
    if ( which > 7 )
      return Drawable::pointer();
    return m_handle_shape[ which ];
  }

  void Handlebox::clear_handles( )
  {
    for ( int i = 0; i < 8; i++ )
      m_handle_shape[ i ].reset();
  }

  void Handlebox::clear_handle( unsigned int which )
  {
    if ( which < 8 )
      m_handle_shape[ which ].reset();
  }

  void Handlebox::position_handle( Drawable::pointer drawable, double x, double y )
  {
    double cx, cy;

    if ( !drawable )
      return ;

    // Get the drawable's bounding box center
    drawable->extents().center( cx, cy );

    // Set the drawable's (x,y) to the desired center, offset by the bounding_box center
    drawable->set_translate( x - cx, y - cy );
    return ;
  }

  int Handlebox::inside_handle( double x, double y )
  {
    for ( int i = 0; i < 8; i++ )
      if ( m_handle_shape[i] && m_handle_shape[i]->inside(x, y) )
        return i;
    return -1;
  }
  
}

